// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;

import javax.swing.SwingUtilities;
import javax.swing.Timer;

import org.apache.log4j.Logger;

import performancemonster.analyzer.LogAnalyzer;
import performancemonster.graph.GraphModel;
import performancemonster.helpers.SystemProperties;
import performancemonster.loghandler.LogServer;
import performancemonster.presentation.overview.MonsterGUI;

public class PerformanceMonster implements ActionListener
{
  private static final Logger logger = Logger
      .getLogger(PerformanceMonster.class);
  private static MonsterGUI gui;

  public static void main(String[] args)
  {
    LogServer server;
    parseArguments(args);
    
    int portnumber = SystemProperties.getInstance().getPortNumber();
    try {
      server = new LogServer(portnumber);
    } catch (IOException e) {
      String errorString = "Error creating server at port " + portnumber;
      logger.error(errorString, e);
      throw new IllegalStateException(errorString, e);
    }

    // The main underlying model
    GraphModel model = new GraphModel();
    
    // Attach a LogAnalyzer to the model, so that the model is updated as events
    // are received over the network
    LogAnalyzer logAnalyzer = new LogAnalyzer(model);
    server.addLogListener(logAnalyzer);
    server.start();

    gui = new MonsterGUI(model);

    // Set up the timer that will perform the continuous updating of the GUI
    Timer timer = new Timer(100, new PerformanceMonster());
    timer.setInitialDelay(1000);
    timer.setCoalesce(false);
    timer.start(); // Start the animation.

    SwingUtilities.invokeLater(new Runnable()
    {
      public void run()
      {
        gui.createAndShowGUI();
      }
    });
  }

  private static void printUsageAndExit()
  {
    System.out
        .println("Usage: java performancemonster.Performancemonster <port>");
    System.out.println(" where <port>: Port number to listen on, e.g. 4446.");
    System.exit(0);
  }

  /**
   * Parses the arguments.
   */
  private static void parseArguments(String[] args)
  {
    SystemProperties systemProperties = SystemProperties.getInstance();

    // First argument is the port number, and is mandatory
    if (args.length < 1) {
      printUsageAndExit();
    }
    try {
      systemProperties.setPortNumber(Integer.parseInt(args[0]));
    } catch (NumberFormatException e) {
      System.out.println("Could not parse port number " + args[0]);
      printUsageAndExit();
    }

    // Further optional arguments follow
    for (int i=1; i<args.length; i++) {
      String arg = args[i];
      if (arg.equals("-instrumentpath")) {
        if (i == args.length - 1) {
          System.out.println("Error: -instrumentpath must be followed by a path");
          printUsageAndExit();
        }
        i++;
        systemProperties.setInstrumentPath(args[i]);
      } else {
        System.out.println("Error: Unsupported argument " + arg);
        printUsageAndExit();
      }
    }
  }
  
  public void actionPerformed(ActionEvent e)
  {
    if (!gui.isFrozen()) {
      gui.update();
    }
  }
}
