// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.graph.coloring;

import java.awt.Color;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.List;
import java.util.Set;

import performancemonster.graph.GraphElement;
import performancemonster.graph.coloring.impl.ColorRuleChangeListener;

public class Colorer
{
  private final Set<ColorChangeListener> listeners;
  private Color backgroundColor;
  private Color defaultBarColor;
  private Color defaultBorderColor;
  private List<ColorRule> colorRules;

  private final ColorRuleChangeListener ruleChangeListener = new ColorRuleChangeListener()
  {
    public void colorRuleChanged()
    {
      notifyListeners();
    }
  };

  public Colorer()
  {
    listeners = new HashSet<ColorChangeListener>();
    setBackgroundColor(Color.WHITE);
    setDefaultBarColor(Color.RED);
    setDefaultBorderColor(Color.BLACK);
    colorRules = new ArrayList<ColorRule>();
  }

  private Colorer(Color backgroundColor, Color defaultBarColor,
      Color defaultBorderColor, List<ColorRule> colorRules)
  {
    this.listeners = new HashSet<ColorChangeListener>();
    this.setBackgroundColor(backgroundColor);
    this.setDefaultBarColor(defaultBarColor);
    this.setDefaultBorderColor(defaultBorderColor);
    this.colorRules = new ArrayList<ColorRule>(colorRules.size());
    for (ColorRule rule : colorRules) {
      appendColorRule(rule.copy());
    }
  }

  /**
   * Creates a copy of this {@link Colorer}, in order to make an "undo"
   * operation possible by using the {@link #replaceWith(Colorer)} method later.
   * 
   * @return an exact copy of this {@link Colorer}, except that listeners are
   *         not copied.
   */
  public Colorer copy()
  {
    return new Colorer(getBackgroundColor(), getDefaultBarColor(),
        getDefaultBorderColor(), colorRules);
  }

  /**
   * Replaces the contents of this {@link Colorer} with the given one, except
   * that listeners are not altered. Listeners of this {@link Colorer} are
   * notified.
   * 
   * @param other
   *          {@link Colorer} to replace contents with.
   */
  public void replaceWith(Colorer other)
  {
    backgroundColor = other.getBackgroundColor();
    defaultBarColor = other.getDefaultBarColor();
    defaultBorderColor = other.getDefaultBorderColor();
    colorRules.clear();
    for (ColorRule rule : other.colorRules) {
      ColorRule copy = rule.copy();
      appendColorRuleNoNotifications(copy);
    }
    notifyListeners();
  }

  /**
   * Finds a bar color for the given {@link GraphElement} from the current
   * rules.
   * 
   * @param element
   *          The {@link GraphElement} to find a bar color for.
   * @return the correct bar color for the element.
   */
  public Color chooseBarColor(GraphElement element)
  {
    for (ColorRule rule : colorRules) {
      if (rule.matches(element)) {
        return rule.getBarColor();
      }
    }
    return getDefaultBarColor();
  }

  /**
   * Finds a border color for the given {@link GraphElement} from the current
   * rules.
   * 
   * @param element
   *          The {@link GraphElement} to find a border color for.
   * @return the correct border color for the element.
   */
  public Color chooseBorderColor(GraphElement element)
  {
    for (ColorRule rule : colorRules) {
      if (rule.matches(element)) {
        return rule.getBorderColor();
      }
    }
    return getDefaultBorderColor();
  }

  public void addChangeListener(ColorChangeListener listener)
  {
    listeners.add(listener);
  }

  public void removeChangeListener(ColorChangeListener listener)
  {
    listeners.remove(listener);
  }

  public void setBackgroundColor(Color backgroundColor)
  {
    this.backgroundColor = backgroundColor;
    notifyListeners();
  }

  public Color getBackgroundColor()
  {
    return backgroundColor;
  }

  public void setDefaultBarColor(Color defaultBarColor)
  {
    this.defaultBarColor = defaultBarColor;
    notifyListeners();
  }

  public Color getDefaultBarColor()
  {
    return defaultBarColor;
  }

  public void setDefaultBorderColor(Color defaultBorderColor)
  {
    this.defaultBorderColor = defaultBorderColor;
    notifyListeners();
  }

  public Color getDefaultBorderColor()
  {
    return defaultBorderColor;
  }

  public List<ColorRule> getColorRules()
  {
    return Collections.unmodifiableList(colorRules);
  }

  public void appendColorRule(ColorRule colorRule)
  {
    appendColorRuleNoNotifications(colorRule);
    notifyListeners();
  }

  public void removeColorRule(ColorRule colorRule)
  {
    if (colorRule == null) {
      return;
    }
    colorRule.removeColorRuleChangeListener(ruleChangeListener);
    colorRules.remove(colorRule);
    notifyListeners();
  }

  public boolean contains(ColorRule colorRule)
  {
    return colorRules.contains(colorRule);
  }

  public void moveUp(ColorRule colorRule)
  {
    if (colorRule == null) {
      return;
    }
    int position = colorRules.indexOf(colorRule);
    if (position == -1) {
      throw new IllegalArgumentException(colorRule + " not contained in Colorer");
    }
    if (position > 0) {
      ColorRule previousRule = colorRules.get(position - 1);
      colorRules.set(position - 1, colorRule);
      colorRules.set(position, previousRule);
      notifyListeners();
    }
  }

  public void moveDown(ColorRule colorRule)
  {
    if (colorRule == null) {
      return;
    }
    int position = colorRules.indexOf(colorRule);
    if (position == -1) {
      throw new IllegalArgumentException(colorRule + " not contained in Colorer");
    }
    if (position < colorRules.size() - 1) {
      ColorRule nextRule = colorRules.get(position + 1);
      colorRules.set(position + 1, colorRule);
      colorRules.set(position, nextRule);
      notifyListeners();
    }
  }
  
  public int getNumberOfRules()
  {
    return colorRules.size();
  }
  
  public ColorRule getColorRule(int index)
  {
    return colorRules.get(index);
  }

  private void notifyListeners()
  {
    for (ColorChangeListener listener : listeners) {
      listener.coloringChanged();
    }
  }

  private void appendColorRuleNoNotifications(ColorRule colorRule)
  {
    colorRule.addColorRuleChangeListener(ruleChangeListener);
    colorRules.add(colorRule);
  }
}
