// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.instrumentation.impl;

import java.io.File;
import java.util.*;

import org.apache.log4j.Logger;
import org.objectweb.asm.*;
import org.objectweb.asm.commons.EmptyVisitor;

import performancemonster.helpers.SystemProperties;
import performancemonster.instrumentation.ClassDefinition;
import performancemonster.instrumentation.MethodDefinition;

public class MethodFinder
{
  private static final Logger logger = Logger.getLogger(MethodFinder.class);
  private static final MethodFinder INSTANCE = new MethodFinder();

  private final Collection<ClassDefinition> foundClasses;

  public static MethodFinder getInstance()
  {
    return INSTANCE;
  }

  private MethodFinder()
  {
    foundClasses = new ArrayList<ClassDefinition>();
    String path = SystemProperties.getInstance().getInstrumentPath();
    for (StringTokenizer tokenizer = new StringTokenizer(path); tokenizer
        .hasMoreElements();) {
      foundClasses.addAll(scanPathEntry(tokenizer.nextToken()));
    }
  }

  private Collection<ClassDefinition> scanPathEntry(String pathEntry)
  {
    File file = new File(pathEntry);
    if (file.isFile()) {
      return JarFileClassDefinition.analyzeJarFile(pathEntry);
    } else if (file.isDirectory()) {
      return DirectoryFileClassDefinition.analyzeDirectory(pathEntry);
    } else {
      logger.warn("Could not determine type of file " + pathEntry);
    }
    return new ArrayList<ClassDefinition>();
  }

  /**
   * Return all found classes containing the given string in its name.
   */
  public Collection<ClassDefinition> doSearch(String searchString)
  {
    Collection<ClassDefinition> result = new ArrayList<ClassDefinition>();
    for (ClassDefinition definition : foundClasses) {
      if (definition.getFullName().contains(searchString)) {
        result.add(definition);
      }
    }
    return result;
  }

  /**
   * Finds all methods in the given chunk of bytes.
   */
  protected static List<MethodDefinition> getMethods(ClassDefinition classDefinition)
  {
    List<MethodDefinition> result = new ArrayList<MethodDefinition>();
    ClassReader classReader = new ClassReader(classDefinition.getBytes());
    ClassVisitor classVisitor = new MethodFinderClassVisitor(result,
        classDefinition);
    classReader.accept(classVisitor, true);
    return result;
  }

  private static class MethodFinderClassVisitor extends ClassAdapter
  {
    private final Collection<MethodDefinition> methods;

    private final ClassDefinition classDefinition;

    public MethodFinderClassVisitor(Collection<MethodDefinition> methods,
        ClassDefinition classDefinition)
    {
      super(new EmptyVisitor());
      this.methods = methods;
      this.classDefinition = classDefinition;
    }

    public MethodVisitor visitMethod(int access, String name, String desc,
        String signature, String[] exceptions)
    {
      methods.add(new MethodDefinitionImpl(name, classDefinition));
      return super.visitMethod(access, name, desc, signature, exceptions);
    }
  }
}
