// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.instrumentation.settings;

import java.util.regex.Pattern;

import performancemonster.instrumentation.ClassDefinition;
import performancemonster.instrumentation.MethodDefinition;

public class ClassSetting implements Setting
{
  public static enum Instrumentation
  {
    inherit, exclude, specify
  };

  public static ClassSetting createDefaultClassSetting(
      ClassDefinition classDefinition)
  {
    return new ClassSetting(classDefinition, Instrumentation.inherit, "");
  }

  private final ClassDefinition classDefinition;
  private final Instrumentation instrumentation;
  private final String methodName;
  private Pattern methodPattern;

  public ClassSetting(ClassDefinition classDefinition,
      Instrumentation instrumentation, String methodName)
  {
    this.classDefinition = classDefinition;
    this.instrumentation = instrumentation;
    this.methodName = methodName;
  }

  public ClassSetting.Instrumentation getInstrumentation()
  {
    return instrumentation;
  }

  public String getMethodName()
  {
    return methodName;
  }

  public ClassDefinition getClassDefinition()
  {
    return classDefinition;
  }

  public boolean isInstrumentationEnabled(MethodDefinition method)
  {
    switch (instrumentation) {
    case inherit:
      throw new IllegalStateException(
          "Settings of type Inherit do not enable or disable instrumentation");
    case exclude:
      return false;
    case specify:
      return matchesMethod(method.getName());
    }
    throw new IllegalStateException("Unknown instrumentation type: "
        + instrumentation);
  }

  public String toString()
  {
    StringBuffer s = new StringBuffer(classDefinition.getFullName());
    if (instrumentation == Instrumentation.inherit) {
      s.append(": inherit");
    } else if (instrumentation == Instrumentation.exclude) {
      s.append(": exclude");
    } else if (instrumentation == Instrumentation.specify) {
      s.append(": methods[");
      s.append(methodName);
      s.append("]");
    }
    return s.toString();
  }

  public int hashCode()
  {
    return classDefinition.hashCode();
  }

  public boolean equals(Object o)
  {
    if (o == this) {
      return true;
    }
    if (o == null) {
      return false;
    }
    if (!o.getClass().equals(this.getClass())) {
      return false;
    }
    ClassSetting other = (ClassSetting) o;
    return classDefinition.equals(other.classDefinition)
        && instrumentation == other.instrumentation
        && methodName.equals(other.methodName);
  }
  
  private boolean matchesMethod(String method)
  {
    if (methodPattern == null) {
      methodPattern = Pattern.compile(methodName);
    }
    return methodPattern.matcher(method).find();
  }
}