// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.instrumentation.settings;

import java.util.regex.Pattern;

import performancemonster.instrumentation.MethodDefinition;

public class PackageSetting implements Setting
{
  public static enum Instrumentation
  {
    inherit, exclude, specify
  };

  public static PackageSetting createDefaultPackageSetting(String packageName)
  {
    return new PackageSetting(packageName, Instrumentation.inherit, "", "");
  }

  private final String packageName;
  private final Instrumentation instrumentation;
  private final String className;
  private final String methodName;
  private Pattern classPattern;
  private Pattern methodPattern;

  public PackageSetting(String packageName, Instrumentation instrumentation,
      String className, String methodName)
  {
    this.packageName = packageName;
    this.instrumentation = instrumentation;
    this.className = className;
    this.methodName = methodName;
  }

  public PackageSetting.Instrumentation getInstrumentation()
  {
    return instrumentation;
  }

  public String getClassName()
  {
    return className;
  }

  public String getMethodName()
  {
    return methodName;
  }

  public String getPackageName()
  {
    return packageName;
  }

  public boolean isInstrumentationEnabled(MethodDefinition method)
  {
    switch (instrumentation) {
    case inherit:
      throw new IllegalStateException(
          "Settings of type Inherit do not enable or disable instrumentation");
    case exclude:
      return false;
    case specify:
      return matchesClass(method.getClassDefinition().getShortName())
          && matchesMethod(method.getName());
    }
    throw new IllegalStateException("Unknown instrumentation type: "
        + instrumentation);
  }

  public String toString()
  {
    StringBuffer s = new StringBuffer(packageName);
    if (instrumentation == Instrumentation.inherit) {
      s.append(": inherit");
    } else if (instrumentation == Instrumentation.exclude) {
      s.append(": exclude");
    } else if (instrumentation == Instrumentation.specify) {
      s.append(": classes[");
      s.append(className);
      s.append("] methods[");
      s.append(methodName);
      s.append("]");
    }
    return s.toString();
  }

  public int hashCode()
  {
    return packageName.hashCode() ^ className.hashCode()
        ^ methodName.hashCode();
  }

  public boolean equals(Object o)
  {
    if (o == this) {
      return true;
    }
    if (o == null) {
      return false;
    }
    if (!o.getClass().equals(this.getClass())) {
      return false;
    }
    PackageSetting other = (PackageSetting) o;
    return packageName.equals(other.packageName)
        && instrumentation == other.instrumentation
        && className.equals(other.className)
        && methodName.equals(other.methodName);
  }

  private boolean matchesClass(String aClass)
  {
    if (classPattern == null) {
      classPattern = Pattern.compile(className);
    }
    return classPattern.matcher(aClass).find();
  }

  private boolean matchesMethod(String method)
  {
    if (methodPattern == null) {
      methodPattern = Pattern.compile(methodName);
    }
    return methodPattern.matcher(method).find();
  }
}