// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.instrumentation;

import java.awt.event.ActionEvent;
import java.util.HashSet;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.InputVerifier;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import org.apache.log4j.Logger;

import performancemonster.instrumentation.InstrumentationFacade;
import performancemonster.instrumentation.exceptions.InstrumentationException;
import performancemonster.instrumentation.settings.Setting;
import performancemonster.presentation.helpers.Dialogs;
import performancemonster.presentation.helpers.LayoutManagerHandler;
import performancemonster.presentation.instrumentation.impl.panels.InstrumentationSelection;
import performancemonster.presentation.instrumentation.impl.panels.InstrumentedList;

public class InstrumentGUI
{
  private static final InstrumentGUI instance = new InstrumentGUI();
  private static final Logger logger = Logger.getLogger(InstrumentGUI.class);
  private final InstrumentationSelection instrumentationSelection;
  private final InstrumentedList instrumentedList;
  private final JFrame frame;
  private final InstrumentationFacade instrumentation;
  private final Dialogs dialogs = Dialogs.getInstance();

  public static InstrumentGUI getInstance()
  {
    return instance;
  }

  private InstrumentGUI()
  {
    instrumentation = new InstrumentationFacade();

    // Create and set up the window.
    frame = new JFrame("Performance Monster - Instrumentation");
    frame.setDefaultCloseOperation(JFrame.HIDE_ON_CLOSE);

    instrumentationSelection = new InstrumentationSelection(instrumentation);
    instrumentedList = new InstrumentedList(instrumentation, instrumentationSelection);
    instrumentationSelection.setInstrumentedList(instrumentedList);
    final JTextField hostnameField = new JTextField(instrumentation
        .getHostName());
    final JTextField portField = new JTextField("" + instrumentation.getPort());
    final JTextField classField = new JTextField();

    hostnameField.setInputVerifier(new InputVerifier()
    {
      public boolean verify(JComponent input)
      {
        JTextField field = (JTextField) input;
        instrumentation.setHostName(field.getText());
        return true;
      }
    });
    portField.setInputVerifier(new InputVerifier()
    {
      public boolean verify(JComponent input)
      {
        JTextField field = (JTextField) input;
        try {
          instrumentation.setPort(Integer.parseInt(field.getText()));
        } catch (NumberFormatException e) {
          logger.info("Invalid port number '" + field.getText() + "'");
          return false;
        }
        return true;
      }
    });

    Action searchClassAction = new AbstractAction("Search")
    {
      public void actionPerformed(ActionEvent evt)
      {
        instrumentationSelection.doSearch(classField.getText());
      }
    };
    JButton searchClassButton = new JButton(searchClassAction);

    Action unInstrumentAction = new AbstractAction("Remove")
    {
      public void actionPerformed(ActionEvent evt)
      {
        // Remove all selected nodes in the instrumented list
        int[] selectedSettings = instrumentedList.getSelectedIndices();
        Set<Setting> settingsToRemove = new HashSet<Setting>();
        for (int i = 0; i < selectedSettings.length; i++) {
          Setting setting = instrumentedList.getElementAt(selectedSettings[i]);
          settingsToRemove.add(setting);
        }
        instrumentation.removeSettings(settingsToRemove);
      }
    };
    Action clearInstrumentationAction = new AbstractAction("Remove all")
    {
      public void actionPerformed(ActionEvent evt)
      {
        // Remove all nodes in the instrumented list
        Set<Setting> settingsToRemove = new HashSet<Setting>();
        for (int i = 0; i < instrumentedList.getSize(); i++) {
          Setting setting = instrumentedList.getElementAt(i);
          settingsToRemove.add(setting);
        }
        instrumentation.removeSettings(settingsToRemove);
      }
    };
    Action updateAction = new AbstractAction("Update")
    {
      public void actionPerformed(ActionEvent evt)
      {
        try {
          instrumentation.updateInstrumentation();
        } catch (InstrumentationException e) {
          String errorString = "Could not update";
          logger.error(errorString, e);
          dialogs.showError(frame, errorString, e);
          return;
        }
      }
    };

    JButton unInstrument = new JButton(unInstrumentAction);
    JButton clearInstrumentation = new JButton(clearInstrumentationAction);
    JButton update = new JButton(updateAction);
    LayoutManagerHandler buttonLayouter = new LayoutManagerHandler(
        LayoutManagerHandler.Direction.HORIZONTAL);
    buttonLayouter.addComponent(unInstrument, true, 1.0);
    buttonLayouter.addComponent(clearInstrumentation, true, 1.0);
    buttonLayouter.addComponent(update, true, 1.0);

    LayoutManagerHandler layouter = new LayoutManagerHandler(
        LayoutManagerHandler.Direction.VERTICAL);
    layouter.addComponent(createHostnamePortPanel(hostnameField, portField),
        false, 0.0);
    layouter.addComponent(createSearchPanel(classField, searchClassButton),
        false, 0.0);
    layouter.addComponent(instrumentationSelection.createPanel(), true, 1.0);
    layouter.addComponent(instrumentedList.createPanel(), true, 1.0);
    layouter.addComponent(buttonLayouter.getPanel(), false, 0.0);

    frame.add(layouter.getPanel());
    frame.pack();
  }

  public void show()
  {
    frame.setVisible(true);
  }

  private JPanel createSearchPanel(JTextField classField,
      JButton searchClassButton)
  {
    LayoutManagerHandler layouter = new LayoutManagerHandler(
        LayoutManagerHandler.Direction.HORIZONTAL);
    layouter.addComponent(classField, true, 1.0);
    layouter.addComponent(searchClassButton, false, 0.0);
    return layouter.getPanel();
  }

  private JPanel createHostnamePortPanel(JTextField hostnameField,
      JTextField portField)
  {
    LayoutManagerHandler layouter = new LayoutManagerHandler(
        LayoutManagerHandler.Direction.HORIZONTAL);
    layouter.addComponent(new JLabel("Host:"), false, 0.0);
    layouter.addComponent(hostnameField, true, 1.0);
    layouter.addComponent(new JLabel("Port:"), false, 0.0);
    layouter.addComponent(portField, true, 0.3);
    return layouter.getPanel();
  }
}
