// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.instrumentation.impl;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.StringTokenizer;

import performancemonster.instrumentation.ClassDefinition;
import performancemonster.instrumentation.InstrumentationFacade;

public class PackageRootNode extends PackageTreeNode
{
  private final InstrumentationFacade instrumentation;
  
  public PackageRootNode(InstrumentationFacade instrumentation)
  {
    super(null, "");
    this.instrumentation = instrumentation;
    doSearch("");
  }
  
  public void doSearch(String searchText)
  {
    clearChildren();
    SubEntries subEntries = new SubEntries("");
    for (ClassDefinition def : instrumentation.findClasses(searchText)) {
      String packageName = def.getPackageName();
      StringTokenizer tokenizer = new StringTokenizer(packageName, ".");
      addClassToEntries(subEntries, def, tokenizer);
    }

    constructNodes(this, subEntries);
  }

  private void constructNodes(PackageTreeNode parent, SubEntries entries)
  {
    // Construct sub-packages
    List<String> subPackages = new ArrayList<String>(entries.getSubPackages());
    Collections.sort(subPackages);
    for (String subPath : subPackages) {
      PackageTreeNode node = new PackageTreeNode(parent, subPath);
      SubEntries subEntries = entries.getPackage(subPath);
      constructNodes(node, subEntries);
      parent.addChild(node);
    }

    // Construct sub-classes
    List<ClassDefinition> subClasses = new ArrayList<ClassDefinition>(entries
        .getSubClasses());
    Collections.sort(subClasses, new Comparator<ClassDefinition>()
    {
      public int compare(ClassDefinition o1, ClassDefinition o2)
      {
        return o1.getShortName().compareTo(o2.getShortName());
      }
    });
    for (ClassDefinition def : subClasses) {
      ClassTreeNode node = new ClassTreeNode(parent, def);
      parent.addChild(node);
    }
  }

  /**
   * Recursively adds package paths and classes to SubEntries.
   */
  private void addClassToEntries(SubEntries entries, ClassDefinition def,
      StringTokenizer tokenizer)
  {
    if (tokenizer.hasMoreElements()) {
      String part = tokenizer.nextToken();
      entries.addPackage(part);
      SubEntries subEntries = entries.getPackage(part);
      addClassToEntries(subEntries, def, tokenizer);
    } else {
      entries.addClass(def);
    }
  }

  private static class SubEntries
  {
    private final String subPath;
    private final Set<ClassDefinition> classes;
    private final Map<String, SubEntries> packages;

    public SubEntries(String subPath)
    {
      this.subPath = subPath;
      classes = new HashSet<ClassDefinition>();
      packages = new HashMap<String, SubEntries>();
    }

    public String getSubPath()
    {
      return subPath;
    }

    public Collection<String> getSubPackages()
    {
      return packages.keySet();
    }

    public Collection<ClassDefinition> getSubClasses()
    {
      return classes;
    }

    public void addClass(ClassDefinition def)
    {
      classes.add(def);
    }

    public void addPackage(String subPath)
    {
      if (!packages.containsKey(subPath)) {
        packages.put(subPath, new SubEntries(subPath));
      }
    }

    public SubEntries getPackage(String subPath)
    {
      return packages.get(subPath);
    }
  }
}
