// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.instrumentation.impl.panels;

import java.awt.CardLayout;
import java.util.Collections;
import java.util.Set;
import java.util.StringTokenizer;

import javax.swing.ButtonGroup;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTextField;
import javax.swing.JTree;
import javax.swing.event.TreeSelectionEvent;
import javax.swing.event.TreeSelectionListener;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeNode;
import javax.swing.tree.TreePath;

import performancemonster.instrumentation.ClassDefinition;
import performancemonster.instrumentation.InstrumentationFacade;
import performancemonster.instrumentation.InstrumentationListener;
import performancemonster.instrumentation.settings.ClassSetting;
import performancemonster.instrumentation.settings.PackageSetting;
import performancemonster.instrumentation.settings.Setting;
import performancemonster.presentation.helpers.LayoutManagerHandler;
import performancemonster.presentation.instrumentation.impl.ClassTreeNode;
import performancemonster.presentation.instrumentation.impl.PackageRootNode;
import performancemonster.presentation.instrumentation.impl.PackageTreeNode;

public class InstrumentationSelection implements InstrumentationListener,
    TreeSelectionListener
{
  private static final String DETAILS_BLANK = "Blank";
  private static final String DETAILS_PACKAGE = "Package";
  private static final String DETAILS_CLASS = "Class";

  private final InstrumentationFacade instrumentation;
  private InstrumentedList instrumentedList;
  private final PackageRootNode packageRootNode;
  private final DefaultTreeModel packageTreeModel;
  private final JTree packageTree;
  private Setting previousSetting;
  private CardLayout cardLayout;
  private JPanel details;

  private JRadioButton classInherit;
  private JRadioButton classExclude;
  private JRadioButton classSpecify;
  private JTextField classMethodName;

  private JRadioButton packageInherit;
  private JRadioButton packageExclude;
  private JRadioButton packageSpecify;
  private JTextField packageClassName;
  private JTextField packageMethodName;

  public InstrumentationSelection(InstrumentationFacade instrumentation)
  {
    this.instrumentation = instrumentation;
    packageRootNode = new PackageRootNode(instrumentation);
    packageTreeModel = new DefaultTreeModel(packageRootNode);
    packageTree = new JTree(packageTreeModel);
    packageTree.setRootVisible(false);
    packageTree.setShowsRootHandles(true);
    packageTree.setExpandsSelectedPaths(true);
  }
  
  public void setInstrumentedList(InstrumentedList instrumentedList)
  {
    this.instrumentedList = instrumentedList;
  }

  /**
   * Search for all classes containing the text in the search text field, and
   * show the result in the package list
   * 
   * @param searchText
   *          The string to search for.
   */
  public void doSearch(String searchText)
  {
    packageRootNode.doSearch(searchText);
    packageTreeModel.reload();
  }

  public JPanel createPanel()
  {
    cardLayout = new CardLayout();
    details = new JPanel(cardLayout);

    // Blank
    details.add(new JPanel(), DETAILS_BLANK);

    // Package options
    {
      LayoutManagerHandler packageLayouter = new LayoutManagerHandler(
          LayoutManagerHandler.Direction.VERTICAL);
      ButtonGroup packageButtonGroup = new ButtonGroup();
      packageInherit = new JRadioButton("Inherit");
      packageExclude = new JRadioButton("Exclude all");
      packageSpecify = new JRadioButton("Specify");
      packageButtonGroup.add(packageInherit);
      packageButtonGroup.add(packageExclude);
      packageButtonGroup.add(packageSpecify);
      packageLayouter.addComponent(packageInherit, false, 0.0);
      packageLayouter.addComponent(packageExclude, false, 0.0);
      packageLayouter.addComponent(packageSpecify, false, 0.0);

      LayoutManagerHandler classNameLayouter = new LayoutManagerHandler(
          LayoutManagerHandler.Direction.HORIZONTAL);
      classNameLayouter.addComponent(new JLabel("Classes:"), false, 0.0);
      packageClassName = new JTextField();
      classNameLayouter.addComponent(packageClassName, true, 1.0);
      packageLayouter.addComponent(classNameLayouter.getPanel(), false, 0.0);

      LayoutManagerHandler methodNameLayouter = new LayoutManagerHandler(
          LayoutManagerHandler.Direction.HORIZONTAL);
      methodNameLayouter.addComponent(new JLabel("Methods:"), false, 0.0);
      packageMethodName = new JTextField();
      methodNameLayouter.addComponent(packageMethodName, true, 1.0);
      packageLayouter.addComponent(methodNameLayouter.getPanel(), false, 0.0);

      details.add(packageLayouter.getPanel(), DETAILS_PACKAGE);
    }

    // Class options
    {
      LayoutManagerHandler classLayouter = new LayoutManagerHandler(
          LayoutManagerHandler.Direction.VERTICAL);
      ButtonGroup classButtonGroup = new ButtonGroup();
      classInherit = new JRadioButton("Inherit");
      classExclude = new JRadioButton("Exclude all");
      classSpecify = new JRadioButton("Specify");
      classButtonGroup.add(classInherit);
      classButtonGroup.add(classExclude);
      classButtonGroup.add(classSpecify);
      classLayouter.addComponent(classInherit, false, 0.0);
      classLayouter.addComponent(classExclude, false, 0.0);
      classLayouter.addComponent(classSpecify, false, 0.0);

      LayoutManagerHandler classNameLayouter = new LayoutManagerHandler(
          LayoutManagerHandler.Direction.HORIZONTAL);
      classNameLayouter.addComponent(new JLabel("Methods:"), false, 0.0);
      classMethodName = new JTextField();
      classNameLayouter.addComponent(classMethodName, true, 1.0);
      classLayouter.addComponent(classNameLayouter.getPanel(), false, 0.0);

      details.add(classLayouter.getPanel(), DETAILS_CLASS);
    }

    LayoutManagerHandler layouter = new LayoutManagerHandler(
        LayoutManagerHandler.Direction.HORIZONTAL);
    layouter.addComponent(new JScrollPane(packageTree), true, 1.0);
    layouter.addComponent(details, true, 0.5);

    packageTree.addTreeSelectionListener(this);

    return layouter.getPanel();
  }

  /**
   * Shows the given setting in the detail panel. Furthermore, if the setting is
   * available in the tree, it will be selected.
   */
  public void showSetting(Setting setting)
  {
    String packageName, className;
    if (setting instanceof ClassSetting) {
      ClassDefinition classDefinition = ((ClassSetting) setting)
          .getClassDefinition();
      packageName = classDefinition.getPackageName();
      className = classDefinition.getShortName();
    } else if (setting instanceof PackageSetting) {
      packageName = ((PackageSetting) setting).getPackageName();
      className = null;
    } else {
      throw new IllegalArgumentException("Unknown Setting type: " + setting);
    }

    PackageTreeNode node = packageRootNode;
    StringTokenizer tokenizer = new StringTokenizer(packageName, ".");
    while (tokenizer.hasMoreTokens() && node != null) {
      String nextToken = tokenizer.nextToken();
      node = findSubPackage(node, nextToken);
    }
    TreeNode endingNode = null;
    if (node != null) {
      if (className == null) {
        endingNode = node;
      } else {
        endingNode = findSubClass(node, className);
      }
    }
    if (endingNode != null) {
      TreePath path = new TreePath(packageTreeModel.getPathToRoot(endingNode));
      packageTree.setSelectionPath(path);
      packageTree.scrollPathToVisible(path);
    }
  }

  private PackageTreeNode findSubPackage(PackageTreeNode node, String subPackage)
  {
    for (int i = 0; i < node.getChildCount(); i++) {
      TreeNode subNode = node.getChildAt(i);
      if (subNode instanceof PackageTreeNode) {
        PackageTreeNode packageSubNode = (PackageTreeNode) subNode;
        if (packageSubNode.getSubPackageName().equals(subPackage)) {
          return packageSubNode;
        }
      }
    }
    return null;
  }

  private ClassTreeNode findSubClass(PackageTreeNode node, String className)
  {
    for (int i = 0; i < node.getChildCount(); i++) {
      TreeNode subNode = node.getChildAt(i);
      if (subNode instanceof ClassTreeNode) {
        ClassTreeNode classSubNode = (ClassTreeNode) subNode;
        if (classSubNode.getClassDefinition().getShortName().equals(className)) {
          return classSubNode;
        }
      }
    }
    return null;
  }

  public void valueChanged(TreeSelectionEvent e)
  {
    handlePreviousNode();
    TreeNode node = (TreeNode) packageTree.getLastSelectedPathComponent();
    handleNewNode(node);
  }

  /**
   * Stores the settings entered by the user.
   */
  private void handlePreviousNode()
  {
    if (previousSetting instanceof PackageSetting) {
      PackageSetting packageSetting = (PackageSetting) previousSetting;
      previousSetting = null;
      if (packageInherit.isSelected()) {
        // No need to store this setting anymore
        instrumentation.removeSettings(Collections
            .singleton((Setting) packageSetting));
      } else if (packageExclude.isSelected()) {
        instrumentation.putPackageSetting(new PackageSetting(packageSetting
            .getPackageName(), PackageSetting.Instrumentation.exclude, "", ""));
      } else if (packageSpecify.isSelected()) {
        instrumentation.putPackageSetting(new PackageSetting(packageSetting
            .getPackageName(), PackageSetting.Instrumentation.specify,
            packageClassName.getText(), packageMethodName.getText()));
      } else {
        throw new IllegalStateException("No package radio button was selected");
      }
    } else if (previousSetting instanceof ClassSetting) {
      ClassSetting classSetting = (ClassSetting) previousSetting;
      previousSetting = null;
      if (classInherit.isSelected()) {
        // No need to store this setting anymore
        instrumentation.removeSettings(Collections
            .singleton((Setting) classSetting));
      } else if (classExclude.isSelected()) {
        instrumentation.putClassSetting(new ClassSetting(classSetting
            .getClassDefinition(), ClassSetting.Instrumentation.exclude, ""));
      } else if (classSpecify.isSelected()) {
        instrumentation.putClassSetting(new ClassSetting(classSetting
            .getClassDefinition(), ClassSetting.Instrumentation.specify,
            classMethodName.getText()));
      } else {
        throw new IllegalStateException("No class radio button was selected");
      }
    } else if (previousSetting != null) {
      throw new IllegalStateException("Unknown previous setting: "
          + previousSetting);
    }
  }

  private void handleNewNode(TreeNode node)
  {
    if (node instanceof PackageTreeNode) {
      String packageName = ((PackageTreeNode) node).getPackageName();
      PackageSetting setting = instrumentation.getPackageSetting(packageName);
      if (setting == null) {
        setting = PackageSetting.createDefaultPackageSetting(packageName);
      }
      handleNewPackageSetting(setting);
      instrumentedList.setSelectedSetting(setting);
    } else if (node instanceof ClassTreeNode) {
      ClassDefinition classDefinition = ((ClassTreeNode) node)
          .getClassDefinition();
      ClassSetting setting = instrumentation.getClassSetting(classDefinition.getFullName());
      if (setting == null) {
        setting = new ClassSetting(classDefinition,
            ClassSetting.Instrumentation.inherit, "");
      }
      handleNewClassSetting(setting);
      instrumentedList.setSelectedSetting(setting);
    } else if (node == null) {
      handleBlankSetting();
    } else {
      throw new IllegalStateException("Unknown node type: " + node);
    }
  }

  private void handleBlankSetting()
  {
    cardLayout.show(details, DETAILS_BLANK);
    previousSetting = null;
  }

  /**
   * Puts the stored settings for the new node into the GUI.
   */
  private void handleNewPackageSetting(PackageSetting packageSetting)
  {
    packageClassName.setText("");
    packageMethodName.setText("");
    PackageSetting.Instrumentation instrumentation = packageSetting
        .getInstrumentation();
    if (instrumentation == PackageSetting.Instrumentation.inherit) {
      packageInherit.setSelected(true);
    } else if (instrumentation == PackageSetting.Instrumentation.exclude) {
      packageExclude.setSelected(true);
    } else if (instrumentation == PackageSetting.Instrumentation.specify) {
      packageSpecify.setSelected(true);
      packageClassName.setText(packageSetting.getClassName());
      packageMethodName.setText(packageSetting.getMethodName());
    } else {
      throw new IllegalStateException("Unknown Instrumentation: "
          + instrumentation);
    }
    cardLayout.show(details, DETAILS_PACKAGE);
    previousSetting = packageSetting;
  }

  private void handleNewClassSetting(ClassSetting classSetting)
  {
    classMethodName.setText("");
    ClassSetting.Instrumentation instrumentation = classSetting
        .getInstrumentation();
    if (instrumentation == ClassSetting.Instrumentation.inherit) {
      classInherit.setSelected(true);
    } else if (instrumentation == ClassSetting.Instrumentation.exclude) {
      classExclude.setSelected(true);
    } else if (instrumentation == ClassSetting.Instrumentation.specify) {
      classSpecify.setSelected(true);
      classMethodName.setText(classSetting.getMethodName());
    }
    cardLayout.show(details, DETAILS_CLASS);
    previousSetting = classSetting;
  }

  public void settingsAdded(Set<Setting> addedSettings)
  {
    // If we are currently editing an added Setting, and the added Setting is
    // different from what we show, we need to change the GUI
    if (previousSetting != null) {
      for (Setting setting : addedSettings) {
        if (setting instanceof PackageSetting
            && previousSetting instanceof PackageSetting) {
          PackageSetting packageSetting = (PackageSetting) setting;
          PackageSetting previousPackageSetting = (PackageSetting) previousSetting;
          if (packageSetting.getPackageName().equals(
              previousPackageSetting.getPackageName())) {
            handleNewPackageSetting(packageSetting);
          }
        } else if (setting instanceof ClassSetting
            && previousSetting instanceof ClassSetting) {
          ClassSetting classSetting = (ClassSetting) setting;
          ClassSetting previousClassSetting = (ClassSetting) previousSetting;
          if (classSetting.getClassDefinition().equals(
              previousClassSetting.getClassDefinition())) {
            handleNewClassSetting(classSetting);
          }
        }
      }
    }
  }

  public void settingsRemoved(Set<Setting> removedSettings)
  {
    // If we are currently editing any of the removed settings, we need to
    // clear the GUI
    if (previousSetting != null) {
      for (Setting setting : removedSettings) {
        if (setting instanceof PackageSetting
            && previousSetting instanceof PackageSetting) {
          PackageSetting packageSetting = (PackageSetting) setting;
          PackageSetting previousPackageSetting = (PackageSetting) previousSetting;
          if (packageSetting.getPackageName().equals(
              previousPackageSetting.getPackageName())) {
            handleNewPackageSetting(PackageSetting
                .createDefaultPackageSetting(packageSetting.getPackageName()));
          }
        } else if (setting instanceof ClassSetting
            && previousSetting instanceof ClassSetting) {
          ClassSetting classSetting = (ClassSetting) setting;
          ClassSetting previousClassSetting = (ClassSetting) previousSetting;
          if (classSetting.getClassDefinition().equals(
              previousClassSetting.getClassDefinition())) {
            handleNewClassSetting(ClassSetting
                .createDefaultClassSetting(classSetting.getClassDefinition()));
          }
        }
      }
    }
  }
}
