// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.presentation.overview;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.io.File;
import java.util.Collections;
import java.util.Set;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.JToggleButton;
import javax.swing.KeyStroke;

import org.apache.log4j.Logger;

import performancemonster.external.GraphExporter;
import performancemonster.external.GraphImporter;
import performancemonster.external.exceptions.ExternalException;
import performancemonster.graph.GraphElement;
import performancemonster.graph.GraphModel;
import performancemonster.graph.coloring.Colorer;
import performancemonster.presentation.colors.ColorGUI;
import performancemonster.presentation.graphelement.GraphElementFrame;
import performancemonster.presentation.helpers.Dialogs;
import performancemonster.presentation.helpers.LookAndFeelHelper;
import performancemonster.presentation.instrumentation.InstrumentGUI;
import performancemonster.presentation.overview.impl.DescriptionListener;
import performancemonster.presentation.overview.impl.GraphArea;
import performancemonster.presentation.overview.impl.HoverListener;

public class MonsterGUI
{
  private static final Logger logger = Logger.getLogger(MonsterGUI.class);
  
  private JToggleButton zoomMode;
  private JToggleButton updateMode;
  private GraphArea graph;
  private JLabel info;
  private JFrame frame;
  private File lastSavedFile;
  private JFileChooser fileChooser;
  private final GraphModel model;
  private final Colorer colorer;

  public MonsterGUI(GraphModel model)
  {
    this.model = model;
    this.colorer = new Colorer();
  }

  /**
   * Create the GUI and show it. For thread safety, this method should be
   * invoked from the event-dispatching thread.
   */
  public void createAndShowGUI()
  {
    LookAndFeelHelper.setCorrectLookAndFeel();

    // Create and set up the window.
    frame = new JFrame("Performance Monster");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

    Action zoomOutAction = new AbstractAction("Zoom out")
    {
      public void actionPerformed(ActionEvent evt)
      {
        long oldBeginning = graph.getBeginning();
        long oldEnd = graph.getEnd();
        long diff = oldEnd - oldBeginning;
        graph.setBeginning(oldBeginning - diff / 2);
        graph.setEnd(oldEnd + diff / 2);
        graph.repaint();
      }
    };
    Action zoomModeAction = new AbstractAction("Zoom mode")
    {
      public void actionPerformed(ActionEvent evt)
      {
        graph.setZoomMode(zoomMode.isSelected());
      }
    };
    Action alignAction = new AbstractAction("Align")
    {
      public void actionPerformed(ActionEvent evt)
      {
        graph.align();
      }
    };
    Action updateAction = new AbstractAction("Update")
    {
      public void actionPerformed(ActionEvent evt)
      {
        // updateMode = updateMode.isSelected();
      }
    };
    Action clearAction = new AbstractAction("Clear")
    {
      public void actionPerformed(ActionEvent evt)
      {
        Set<GraphElement> empty = Collections.emptySet();
        model.clear();
        graph.setGraphElements(empty);
        graph.repaint();
      }
    };
    Action instrumentAction = new AbstractAction("Instrument")
    {
      public void actionPerformed(ActionEvent evt)
      {
        InstrumentGUI.getInstance().show();
      }
    };
    Action colorAction = new AbstractAction("Color")
    {
      public void actionPerformed(ActionEvent evt)
      {
        new ColorGUI(colorer).show();
      }
    };

    JButton zoomOut = new JButton(zoomOutAction);
    zoomMode = new JToggleButton(zoomModeAction);
    JButton reset = new JButton(alignAction);
    reset.setName("AlignButton");
    updateMode = new JToggleButton(updateAction);
    JButton clear = new JButton(clearAction);
    JButton instrument = new JButton(instrumentAction);
    JButton color = new JButton(colorAction);
    info = new JLabel("");
    graph = new GraphArea(colorer);

    graph.setBeginning(0L);
    graph.setEnd(300L);
    graph.addDescriptionListener(new DescriptionListener()
    {
      public void showDescription(GraphElement element)
      {
        new GraphElementFrame(element);
      }
    });
    graph.addHoverListener(new HoverListener()
    {
      public void mouseHovered(long position, GraphElement element)
      {
        info.setText("Position " + (position - graph.getBeginning()) + "ms: "
            + element);
      }
    });

    JPanel panel = new JPanel();
    panel.setLayout(new BorderLayout());

    JPanel controls = new JPanel();
    controls.setLayout(new BoxLayout(controls, BoxLayout.LINE_AXIS));
    controls.add(zoomOut);
    controls.add(zoomMode);
    controls.add(reset);
    controls.add(updateMode);
    controls.add(clear);
    controls.add(instrument);
    controls.add(color);
    controls.add(info);

    panel.add(controls, BorderLayout.SOUTH);
    panel.add(graph, BorderLayout.CENTER);
    frame.add(panel);

    createMenus();
    
    // Display the window.
    frame.pack();
    frame.setVisible(true);
  }
  
  public JFrame getJFrame()
  {
    return frame;
  }
  
  public void update()
  {
    Set<GraphElement> updatedList = model.getElements();
    graph.setGraphElements(updatedList);
    graph.setEnd(System.currentTimeMillis());
    graph.align();
    graph.setEnd(System.currentTimeMillis());
  }

  public boolean isFrozen()
  {
    return !updateMode.isSelected();
  }

  private void createMenus()
  {
    JMenuBar menuBar = new JMenuBar();
    JMenu fileMenu = new JMenu("File");
    fileMenu.setMnemonic(KeyEvent.VK_F);
    menuBar.add(fileMenu);
    
    // File->Load Ctrl-L
    JMenuItem openItem = new JMenuItem("Open", KeyEvent.VK_O);
    openItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_O, ActionEvent.CTRL_MASK));
    openItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        openAction();
      }
    });
    fileMenu.add(openItem);

    // File->Save Ctrl-S
    JMenuItem saveItem = new JMenuItem("Save", KeyEvent.VK_S);
    saveItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_S, ActionEvent.CTRL_MASK));
    saveItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        saveAction();
      }
    });
    fileMenu.add(saveItem);
    
    // File->Save As...
    JMenuItem saveAsItem = new JMenuItem("Save as...");
    saveAsItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        saveAsAction();
      }
    });
    fileMenu.add(saveAsItem);

    fileMenu.addSeparator();
    
    // File->Quit Ctrl-Q
    JMenuItem quitItem = new JMenuItem("Quit");
    quitItem.setAccelerator(KeyStroke.getKeyStroke(
        KeyEvent.VK_Q, ActionEvent.CTRL_MASK));
    quitItem.addActionListener(new ActionListener() {
      public void actionPerformed(ActionEvent e)
      {
        quitAction();
      }
    });
    fileMenu.add(quitItem);
    
    frame.setJMenuBar(menuBar);
  }

  private void openAction()
  {
    if (getFileChooser().showOpenDialog(frame) == JFileChooser.APPROVE_OPTION) {
      File fileToOpen = getFileChooser().getSelectedFile();
      try {
        GraphImporter importer = new GraphImporter(model);
        importer.importFromFile(fileToOpen);
        lastSavedFile = fileToOpen;
        update();
        graph.align();
      } catch (ExternalException e) {
        String errorString = "Could not open file " + fileToOpen;
        logger.error(errorString, e);
        Dialogs.getInstance().showError(frame, errorString, e);
      }
    }
  }

  private void saveAction()
  {
    if (lastSavedFile == null) {
      saveAsAction();
    } else {
      saveToFile(lastSavedFile);
    }
  }
  
  private void saveAsAction()
  {
    if (getFileChooser().showSaveDialog(frame) == JFileChooser.APPROVE_OPTION) {
      File chosenFile = getFileChooser().getSelectedFile();
      if (chosenFile.exists()
          && !Dialogs.getInstance().showYesNo(
              frame,
              "The chosen file " + chosenFile.getName()
                  + " already exists.\nDo you want to overwrite?",
              "File already exists", "Yes", "No")) {
        return;
      }
      lastSavedFile = chosenFile;
      saveToFile(lastSavedFile);
    }
  }

  private void quitAction()
  {
    System.exit(0);
  }

  private void saveToFile(File file)
  {
    try {
      GraphExporter exporter = new GraphExporter(model);
      exporter.exportToFile(file);
    } catch (ExternalException e) {
      String errorString = "Could not save to file " + file;
      logger.error(errorString, e);
      Dialogs.getInstance().showError(frame, errorString, e);
    }
  }

  private JFileChooser getFileChooser()
  {
    if (fileChooser == null) {
      fileChooser = new JFileChooser();
    }
    return fileChooser;
  }
}
