// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.test.external;

import java.awt.Color;
import java.io.File;
import java.io.FileInputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;

import junit.framework.TestCase;
import performancemonster.external.ColorExporter;
import performancemonster.graph.coloring.Colorer;
import performancemonster.helpers.MockFactory;
import performancemonster.schemas.colors.Graphcolors;
import performancemonster.schemas.colors.RuleType;

/**
 * Tests {@link ColorExporter}. Does so by exporting to a file, and then
 * reading the file again using the generated JAXB classes. Is that cheating?
 */
public class ColorExportTest extends TestCase
{
  /**
   * Exports a color model with no rules.
   */
  public void testColorExportNoRules() throws Exception
  {
    File file = new File("test/output/ColorExportTest_NoRules.xml");
    Colorer colorer = new Colorer();
    Color background = new Color(0xFF, 0xAB, 0x10);
    Color defaultBar = new Color(0, 1, 2);
    Color defaultBorder = new Color(7, 8, 9);
    colorer.setBackgroundColor(background);
    colorer.setDefaultBarColor(defaultBar);
    colorer.setDefaultBorderColor(defaultBorder);
    ColorExporter exporter = new ColorExporter(colorer);
    exporter.exportToFile(file);

    Graphcolors colors = loadFromFile(file);

    assertEquals("Wrong number of rules", 0, colors.getRule().size());
    assertEquals("Wrong background color", "#FFAB10", colors.getBackground());
    assertEquals("Wrong default bar color", "#000102", colors.getDefaultBar());
    assertEquals("Wrong default border color", "#070809", colors
        .getDefaultBorder());
  }

  /**
   * Exports a color model with only one rule, checks that all attributes of
   * that rule is correctly exported.
   */
  public void testColorExportOneRule() throws Exception
  {
    File file = new File("test/output/ColorExportTest_OneRule.xml");
    Colorer colorer = new Colorer();
    Color barColor = new Color(0xFF, 0x10, 0x1F);
    Color borderColor = new Color(0, 1, 2);
    colorer.appendColorRule(MockFactory.createColorRule("A test rule",
        barColor, borderColor, "A test category", "A test message",
        "A test stack trace\nwith some\nlines"));
    ColorExporter exporter = new ColorExporter(colorer);
    exporter.exportToFile(file);

    Graphcolors colors = loadFromFile(file);
    assertEquals("Wrong number of rules in the list", 1, colors.getRule()
        .size());
    RuleType element = (RuleType) colors.getRule().get(0);
    assertRuleType(element, "A test rule", "#FF101F", "#000102",
        "A test category", "A test message",
        "A test stack trace\nwith some\nlines");
  }

  /**
   * Exports a model with a single element which has as many of its attributes
   * as possible "blanked out".
   */
  public void testColorExportRuleMissingInformation() throws Exception
  {
    File file = new File(
        "test/output/ColorExportTest_RuleMissingInformation.xml");
    Colorer colorer = new Colorer();
    Color barColor = new Color(0xFF, 0x10, 0x1F);
    Color borderColor = new Color(0, 1, 2);
    colorer.appendColorRule(MockFactory.createColorRule("", barColor,
        borderColor, "", "", ""));
    ColorExporter exporter = new ColorExporter(colorer);
    exporter.exportToFile(file);

    Graphcolors colors = loadFromFile(file);
    assertEquals("Wrong number of rules in the list", 1, colors.getRule()
        .size());
    RuleType element = (RuleType) colors.getRule().get(0);
    assertRuleType(element, "", "#FF101F", "#000102", null, null, null);
  }

  /**
   * Exports a model with 5 rules. Only checks that 5 rules are exported, not
   * that the contents are correct.
   */
  public void testColorExportMoreRules() throws Exception
  {
    File file = new File("test/output/ColorExportTest_MoreRules.xml");
    Colorer colorer = new Colorer();
    colorer.appendColorRule(MockFactory.createColorRule("Rule1", Color.RED,
        Color.BLACK, "Category1", "Message1", "StackTrace1"));
    colorer.appendColorRule(MockFactory.createColorRule("Rule2", Color.RED,
        Color.BLACK, "Category2", "Message2", "StackTrace2"));
    colorer.appendColorRule(MockFactory.createColorRule("Rule3", Color.RED,
        Color.BLACK, "Category3", "Message3", "StackTrace3"));
    colorer.appendColorRule(MockFactory.createColorRule("Rule4", Color.RED,
        Color.BLACK, "Category4", "Message4", "StackTrace4"));
    colorer.appendColorRule(MockFactory.createColorRule("Rule5", Color.RED,
        Color.BLACK, "Category5", "Message5", "StackTrace5"));
    ColorExporter exporter = new ColorExporter(colorer);
    exporter.exportToFile(file);

    Graphcolors colors = loadFromFile(file);
    assertEquals("Wrong number of rules", 5, colors.getRule().size());
  }

  /**
   * Helper for checking the contents of a {@link RuleType}.
   */
  private void assertRuleType(RuleType element, String name, String barColor,
      String borderColor, String category, String message, String stackTrace)
  {
    assertEquals("Name is not correct", name, element.getName());
    assertEquals("Bar color is not correct", barColor, element.getBar());
    assertEquals("Border color is not correct", borderColor, element
        .getBorder());
    assertEquals("Category is not correct", category, element.getCategory());
    assertEquals("Message is not correct", message, element.getMessage());
    assertEquals("Stack trace is not correct", stackTrace, element
        .getStackTrace());
  }

  private Graphcolors loadFromFile(File file) throws Exception
  {
    JAXBContext context = JAXBContext
        .newInstance("performancemonster.schemas.colors");
    Unmarshaller unmarshaller = context.createUnmarshaller();
    return (Graphcolors) unmarshaller.unmarshal(new FileInputStream(file));
  }
}
