// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.test.external;

import java.io.File;
import java.io.FileInputStream;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.Unmarshaller;

import junit.framework.TestCase;
import performancemonster.external.GraphExporter;
import performancemonster.graph.GraphModel;
import performancemonster.helpers.MockFactory;
import performancemonster.schemas.graph.GraphElementType;
import performancemonster.schemas.graph.Graphelements;

/**
 * Tests {@link GraphExporter}. Does so by exporting to a file, and then
 * reading the file again using the generated JAXB classes. Is that cheating?
 */
public class GraphExportTest extends TestCase
{
  /**
   * Exports an empty model.
   */
  public void testGraphExportNoElements() throws Exception
  {
    File file = new File("test/output/GraphExportTest_NoElements.xml");
    GraphModel model = new GraphModel();
    GraphExporter exporter = new GraphExporter(model);
    exporter.exportToFile(file);

    Graphelements graphElements = loadFromFile(file);
    assertEquals("Wrong number of elements in the list", 0, graphElements
        .getGraphelement().size());
  }

  /**
   * Exports a model with only one element, checks that all attributes of that
   * element is correctly exported.
   */
  public void testGraphExportOneElement() throws Exception
  {
    File file = new File("test/output/GraphExportTest_OneElement.xml");
    GraphModel model = new GraphModel();
    model.putElement(MockFactory.createGraphElement("Category1", "Message1",
        "StackTrace1", 10, 20));
    GraphExporter exporter = new GraphExporter(model);
    exporter.exportToFile(file);

    Graphelements graphElements = loadFromFile(file);
    assertEquals("Wrong number of elements in the list", 1, graphElements
        .getGraphelement().size());
    GraphElementType element = (GraphElementType) graphElements
        .getGraphelement().get(0);
    assertGraphElementType(element, "Category1", "Message1", "StackTrace1", 10,
        20);
  }

  /**
   * Exports a model with a single element which has as many of its attributes
   * as possible "blanked out".
   */
  public void testGraphExportElementMissingInformation() throws Exception
  {
    File file = new File(
        "test/output/GraphExportTest_ElementMissingInformation.xml");
    GraphModel model = new GraphModel();
    model.putElement(MockFactory.createGraphElement("", "", null, 0, 0));
    GraphExporter exporter = new GraphExporter(model);
    exporter.exportToFile(file);

    Graphelements graphElements = loadFromFile(file);
    assertEquals("Wrong number of elements in the list", 1, graphElements
        .getGraphelement().size());
    GraphElementType element = (GraphElementType) graphElements
        .getGraphelement().get(0);
    assertGraphElementType(element, "", "", null, 0, 0);
  }

  /**
   * Exports a model with 5 elements. Only checks that 5 elements are exported,
   * not that the contents are correct.
   */
  public void testGraphExportMoreElements() throws Exception
  {
    File file = new File("test/output/GraphExportTest_MoreElements.xml");
    GraphModel model = new GraphModel();
    model.putElement(MockFactory.createGraphElement("Category1", "Message1",
        "StackTrace1", 10, 20));
    model.putElement(MockFactory.createGraphElement("Category2", "Message2",
        "StackTrace2", 20, 340));
    model.putElement(MockFactory.createGraphElement("Category3", "Message3",
        "StackTrace3", 30, 10));
    model.putElement(MockFactory.createGraphElement("Category4", "Message4",
        "StackTrace4", 40, 202));
    model.putElement(MockFactory.createGraphElement("Category5", "Message5",
        "StackTrace5", 1000, 2560));
    GraphExporter exporter = new GraphExporter(model);
    exporter.exportToFile(file);

    Graphelements graphElements = loadFromFile(file);
    assertEquals("Wrong number of elements in the list", 5, graphElements
        .getGraphelement().size());
  }

  /**
   * Helper for checking the contents of a {@link GraphElementType}.
   */
  private void assertGraphElementType(GraphElementType element,
      String category, String message, String stackTrace, long begin, long end)
  {
    assertEquals("Category is not correct", category, element.getCategory());
    assertEquals("Message is not correct", message, element.getMessage());
    assertEquals("Stack trace is not correct", stackTrace, element
        .getStackTrace());
    assertEquals("Begin time is not correct", begin, element.getStartTime()
        .longValue());
    assertEquals("End time is not correct", end, element.getEndTime()
        .longValue());
  }

  private Graphelements loadFromFile(File file) throws Exception
  {
    JAXBContext context = JAXBContext
        .newInstance("performancemonster.schemas.graph");
    Unmarshaller unmarshaller = context.createUnmarshaller();
    return (Graphelements) unmarshaller.unmarshal(new FileInputStream(file));
  }
}
