// PerformanceMonster
// Copyright (C) 2005  Ole Friis <olefriis@gmail.com>
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

package performancemonster.test.graph.coloring;

import static org.easymock.EasyMock.*;

import java.awt.Color;

import junit.framework.TestCase;
import performancemonster.graph.coloring.ColorRule;
import performancemonster.graph.coloring.impl.ColorRuleChangeListener;
import performancemonster.helpers.MockFactory;

public class ColorRuleTest extends TestCase
{
  public void testEmptyRule()
  {
    ColorRule emptyRule = new ColorRule("Empty rule");
    assertTrue("Rule must match a graph element with empty text", emptyRule
        .matches(MockFactory.createGraphElement("", "", "", 0, 0)));
    assertTrue("Rule must match any graph element", emptyRule
        .matches(MockFactory.createGraphElement("Category", "Message",
            "Stack trace", 0, 100)));
  }

  public void testRuleWithCategory()
  {
    ColorRule ruleWithCategory = new ColorRule("Rule with category");
    assertEquals("Default category text must be empty", "", ruleWithCategory
        .getCategory());

    ruleWithCategory.setCategory("category text");
    assertEquals("Set category must be reflected in getter", "category text",
        ruleWithCategory.getCategory());
    assertTrue("Rule must match element with correct category text",
        ruleWithCategory.matches(MockFactory.createGraphElement(
            "A category text", "Message", "Stack trace", 0, 100)));
    assertFalse("Rule must not match element with wrong category text",
        ruleWithCategory.matches(MockFactory.createGraphElement("Other text",
            "Message", "Stack trace", 0, 100)));

    // Redefine text, try again
    ruleWithCategory.setCategory("Other text");
    assertEquals("Set category must be reflected in getter", "Other text",
        ruleWithCategory.getCategory());
    assertFalse("Rule must not match element with wrong category text",
        ruleWithCategory.matches(MockFactory.createGraphElement(
            "A category text", "Message", "Stack trace", 0, 100)));
    assertTrue("Rule must match element with correct category text",
        ruleWithCategory.matches(MockFactory.createGraphElement("Other text",
            "Message", "Stack trace", 0, 100)));
  }

  public void testRuleWithMessage()
  {
    ColorRule ruleWithMessage = new ColorRule("Rule with message");
    assertEquals("Default message text must be empty", "", ruleWithMessage
        .getMessage());

    ruleWithMessage.setMessage("message text");
    assertEquals("Set message must be reflected in getter", "message text",
        ruleWithMessage.getMessage());
    assertTrue("Rule must match element with correct message text",
        ruleWithMessage.matches(MockFactory.createGraphElement(
            "Category", "A message text", "Stack trace", 0, 100)));
    assertFalse("Rule must not match element with wrong message text",
        ruleWithMessage.matches(MockFactory.createGraphElement("Category",
            "Other message", "Stack trace", 0, 100)));

    // Redefine text, try again
    ruleWithMessage.setMessage("Other message");
    assertEquals("Set message must be reflected in getter", "Other message",
        ruleWithMessage.getMessage());
    assertFalse("Rule must not match element with wrong message text",
        ruleWithMessage.matches(MockFactory.createGraphElement(
            "Category", "A message text", "Stack trace", 0, 100)));
    assertTrue("Rule must match element with correct message text",
        ruleWithMessage.matches(MockFactory.createGraphElement("Category",
            "Other message", "Stack trace", 0, 100)));
  }

  public void testRuleWithStackTrace()
  {
    ColorRule ruleWithStackTrace = new ColorRule("Rule with stack trace");
    assertEquals("Default stack trace must be empty", "", ruleWithStackTrace
        .getStackTrace());

    ruleWithStackTrace.setStackTrace("stack trace");
    assertEquals("Set stack trace must be reflected in getter", "stack trace",
        ruleWithStackTrace.getStackTrace());
    assertTrue("Rule must match element with correct stack trace",
        ruleWithStackTrace.matches(MockFactory.createGraphElement("Category",
            "Message", "A stack trace", 0, 100)));
    assertFalse("Rule must not match element with wrong stack trace",
        ruleWithStackTrace.matches(MockFactory.createGraphElement("Category",
            "Message", "Another trace", 0, 100)));

    // Redefine stack trace, try again
    ruleWithStackTrace.setStackTrace("other trace");
    assertEquals("Set stack trace must be reflected in getter", "other trace",
        ruleWithStackTrace.getStackTrace());
    assertFalse("Rule must not match element with wrong stack trace",
        ruleWithStackTrace.matches(MockFactory.createGraphElement("Category",
            "Message", "A stack trace", 0, 100)));
    assertTrue("Rule must match element with correct stack trace",
        ruleWithStackTrace.matches(MockFactory.createGraphElement("Category",
            "Message", "Another trace", 0, 100)));
  }

  public void testNotificationSetBarColor()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);

    listener.colorRuleChanged();
    replay(listener);
    rule.setBarColor(Color.RED);
    verify(listener);
  }

  public void testNotificationSetBorderColor()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);

    listener.colorRuleChanged();
    replay(listener);
    rule.setBorderColor(Color.RED);
    verify(listener);
  }

  public void testNotificationSetCategory()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);

    listener.colorRuleChanged();
    replay(listener);
    rule.setCategory("Dummy category");
    verify(listener);
  }

  public void testNotificationSetMessage()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);

    listener.colorRuleChanged();
    replay(listener);
    rule.setMessage("Dummy message");
    verify(listener);
  }

  public void testNotificationSetName()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);

    listener.colorRuleChanged();
    replay(listener);
    rule.setName("Dummy name");
    verify(listener);
  }

  public void testNotificationSetStackTrace()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);

    listener.colorRuleChanged();
    replay(listener);
    rule.setStackTrace("Dummy stack trace");
    verify(listener);
  }

  public void testNotificationNotOnCopy()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRule ruleCopy = rule.copy();
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    ColorRuleChangeListener listenerCopy = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);
    ruleCopy.addColorRuleChangeListener(listenerCopy);

    // First: Change "rule"
    listener.colorRuleChanged();
    replay(listener);
    replay(listenerCopy);
    rule.setName("Dummy name");
    verify(listener);
    verify(listenerCopy);
    
    reset(listener);
    reset(listenerCopy);
    
    // Then: Change "ruleCopy"
    listenerCopy.colorRuleChanged();
    replay(listener);
    replay(listenerCopy);
    ruleCopy.setName("Dummy copy name");
    verify(listener);
    verify(listenerCopy);
  }

  public void testNotificationSeparatedWhenCopied()
  {
    ColorRule rule = new ColorRule("Rule");
    ColorRuleChangeListener listener = createMock(ColorRuleChangeListener.class);
    rule.addColorRuleChangeListener(listener);

    ColorRule ruleCopy = rule.copy();
    ColorRuleChangeListener listenerCopy = createMock(ColorRuleChangeListener.class);
    ruleCopy.addColorRuleChangeListener(listenerCopy);

    // First: Change "rule"
    listener.colorRuleChanged();
    replay(listener);
    replay(listenerCopy);
    rule.setName("Dummy name");
    verify(listener);
    verify(listenerCopy);

    reset(listener);
    reset(listenerCopy);

    // Then: Change "ruleCopy"
    listenerCopy.colorRuleChanged();
    replay(listener);
    replay(listenerCopy);
    ruleCopy.setName("Dummy copy name");
    verify(listener);
    verify(listenerCopy);
  }
}
